
import os
import io
from redis import Redis
import urllib.parse
from sqlalchemy import create_engine, Column
from sqlalchemy.orm import sessionmaker, declarative_base
from sqlalchemy.dialects.postgresql import UUID, JSON, BYTEA
import matplotlib.pyplot as plt
import uhepp


def get_redis():
    """Return redis connection from env vars"""
    host = os.environ.get("REDIS_HOST", "localhost")
    username = os.environ.get("REDIS_USERNAME", "worker")
    password = os.environ.get("REDIS_PASSWORD", "not-so-secret-worker")

    redis = Redis(host=host, username=username, password=password)
    return redis

def get_psql():
    """Return SQLAlchemy session to PostgreSQL database"""
    host = os.environ.get("POSTGRES_HOST", "localhost")
    user = os.environ.get("POSTGRES_USER", "webapp")
    password = os.environ.get("POSTGRES_PASSWORD", "not-so-secret-db")
    database = os.environ.get("POSTGRES_DB", "uhepp_hub")

    host = urllib.parse.quote_plus(host)
    user = urllib.parse.quote_plus(user)
    password = urllib.parse.quote_plus(password)
    database = urllib.parse.quote_plus(database)

    engine = create_engine(f"postgresql://{user}:{password}@{host}/{database}")
    return sessionmaker(bind=engine)()

Base = declarative_base()
class UheppPlot(Base):
    """Minimal mapping of uhepp plot in PostgreSQL database"""
    __tablename__ = 'uhepp_vault_plot'

    uuid = Column(UUID(), primary_key=True)
    uhepp = Column(JSON())
    preview = Column(BYTEA())

def worker():
    """Loop: Wait for job, render plot, store in database"""
    redis = get_redis()
    psql = get_psql()

    # Harakiri
    for i in range(100):
        _, uuid = redis.blpop("preview_job_queue", timeout=0)
        uuid = uuid.decode()
        print("Working on %s" % uuid)
        uhepp_obj = psql.query(UheppPlot).filter(UheppPlot.uuid == uuid).first()

        try:
            plot = uhepp.from_data(uhepp_obj.uhepp)
            fig, ax = plot.render()

            buf = io.BytesIO()
            fig.savefig(buf, format="png", dpi=30)

            uhepp_obj.preview = buf.getvalue()
        except Exception as e:
            uhepp_obj.preview = b"!error"

        psql.add(uhepp_obj)
        psql.commit()

        plt.close("all")

if __name__ == "__main__":
    worker()

