import math
from django.shortcuts import render
from django.views import generic
from django.db.models import Q
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import JsonResponse, HttpResponseRedirect, \
                        HttpResponseForbidden, Http404, \
                        HttpResponse
from django.urls import reverse, reverse_lazy
from django.shortcuts import get_object_or_404, redirect, render
from django.conf import settings

from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User, Group
from django.templatetags.static import static
from django.utils.translation import gettext_lazy as _

from uhepp_api.models import Token
from uhepp_api.views import MaskedRelatedMixin
from uhepp_api.masks import MaskRelatedColletions
from .models import PUBLIC_LEVEL, INTERNAL_LEVEL, PUBLIC_LEVEL, \
                    Collection, Plot, Profile

from .forms import TokenForm, PlotForm

def quantize(value, sig_digits):
    if value <= 100:
        return value

    magnitude = math.floor(math.log10(value))
    scale = magnitude - sig_digits + 1
    return math.floor(value / 10**scale) * 10**scale
    

def home(request):
    plot_count = Plot.objects.count()
    plot_count = quantize(plot_count, 2)
    context = dict(plot_count=plot_count)
    return render(request, "uhepp_vault/home.html", context=context)

def legal_notice(request):
    return render(request, "uhepp_vault/legal_notice.html")

def privacy_policy(request):
    return render(request, "uhepp_vault/privacy_policy.html")

def terms_of_service(request):
    return render(request, "uhepp_vault/terms_of_service.html")

class TokenListView(LoginRequiredMixin, generic.ListView):
    model = Token
    template_name = 'uhepp_vault/token_list.html'

    def get_queryset(self):
        queryset = Token.objects.filter(user=self.request.user)
        return queryset

class TokenCreateView(LoginRequiredMixin, generic.CreateView):
    model = Token
    template_name = 'uhepp_vault/token_form.html'
    fields = ['description', 'expires']

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.user = self.request.user
        self.object.save()
        return self.render_to_response(self.get_context_data(token=self.object))

class TokenDeleteView(LoginRequiredMixin, generic.DeleteView):
    model = Token
    template_name = "uhepp_vault/token_confirm_delete.html"
    success_url = reverse_lazy('uhepp_vault:token-list')

    def get_queryset(self):
        queryset = Token.objects.filter(user=self.request.user)
        return queryset

class AccountEditView(LoginRequiredMixin, generic.UpdateView):
    model = Profile
    fields = ['visibility']
    success_url = reverse_lazy('uhepp_vault:account-detail')

    def get_object(self, queryset=None):
        return self.request.user.profile

class UserListView(generic.ListView):
    model = Profile

    def get_queryset(self):
        if self.request.user.is_anonymous:
            queryset = Profile.objects.filter(visibility=PUBLIC_LEVEL)
        else:
            queryset = Profile.objects.filter(
                Q(visibility__gte=INTERNAL_LEVEL) |
                Q(user=self.request.user)
            )
        return queryset

class UserDetailView(MaskedRelatedMixin, generic.DetailView):
    model = User
    slug_field = 'username'
    slug_url_kwarg = 'username'
    template_name = "uhepp_vault/profile_detail.html"
    masking_backends = [MaskRelatedColletions]
    context_object_name = "user"

    def get_queryset(self):
        if self.request.user.is_anonymous:
            queryset = User.objects.filter(profile__visibility=PUBLIC_LEVEL)
        else:
            queryset = User.objects.filter(
                (Q(profile__visibility__gte=INTERNAL_LEVEL) |
                Q(pk=self.request.user.pk)) &
                Q(profile__isnull=False)
            )
        return queryset

    def get(self, request, *args, **kwargs):
        try:
            return super().get(request, *args, **kwargs)
        except Http404 as original:
            if not request.user.is_anonymous:
                raise original
            return redirect('%s?next=%s' % (settings.LOGIN_URL, request.path))

class CollectionListView(generic.ListView):
    model = Collection

    def get_queryset(self):
        if self.request.user.is_anonymous:
            queryset = Collection.objects.filter(visibility=PUBLIC_LEVEL)
        else:
            queryset = Collection.objects.filter(
                Q(visibility__gte=INTERNAL_LEVEL) |
                Q(owner=self.request.user)
            )
        return queryset

class CollectionDetailView(generic.DetailView):
    model = Collection

    def get_queryset(self):
        if self.request.user.is_anonymous:
            queryset = Collection.objects.filter(visibility=PUBLIC_LEVEL)
        else:
            queryset = Collection.objects.filter(
                Q(visibility__gte=INTERNAL_LEVEL) |
                Q(owner=self.request.user)
            )
        return queryset

    def get(self, request, *args, **kwargs):
        try:
            return super().get(request, *args, **kwargs)
        except Http404 as original:
            if not request.user.is_anonymous:
                raise original
            return redirect('%s?next=%s' % (settings.LOGIN_URL, request.path))

class CollectionCreateView(LoginRequiredMixin, generic.CreateView):
    model = Collection
    template_name = 'uhepp_vault/collection_form.html'
    fields = ['title', 'description', 'visibility']

    def get_success_url(self):
        return  reverse_lazy('uhepp_vault:collection-detail',
                             args=[self.object.pk])

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.owner = self.request.user
        self.object.save()
        return super().form_valid(form)

class CollectionUpdateView(LoginRequiredMixin, generic.UpdateView):
    model = Collection
    template_name = 'uhepp_vault/collection_form.html'
    fields = ['title', 'description', 'visibility']
    edit = True

    def get_success_url(self):
        return  reverse_lazy('uhepp_vault:collection-detail',
                             args=[self.object.pk])
    def get_queryset(self):
        queryset = Collection.objects.filter(
            owner=self.request.user
        )
        return queryset

class CollectionDeleteView(LoginRequiredMixin, generic.DeleteView):
    model = Collection

    def get_success_url(self):
        return  reverse_lazy('uhepp_vault:user-detail',
                             args=[self.request.user.username])

    def get_queryset(self):
        queryset = Collection.objects.filter(
            owner=self.request.user
        )
        return queryset


class PlotListView(generic.ListView):
    model = Plot

    def get_queryset(self):
        if self.request.user.is_anonymous:
            queryset = Plot.objects.filter(collection__visibility=PUBLIC_LEVEL)
        else:
            queryset = Plot.objects.filter(
                Q(collection__visibility__gte=INTERNAL_LEVEL) |
                Q(collection__owner=self.request.user)
            )
        return queryset

class PlotDetail(generic.UpdateView):
    model = Plot
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'
    fields = ["comment"]
    template_name = 'uhepp_vault/plot_detail.html'

    def get_queryset(self):
        if self.request.user.is_anonymous:
            queryset = Plot.objects.filter(collection__visibility=PUBLIC_LEVEL)
        else:
            queryset = Plot.objects.filter(
                Q(collection__visibility__gte=INTERNAL_LEVEL) |
                Q(collection__owner=self.request.user)
            )
        return queryset

    def get_object(self, *args, **kwds):
        obj = super().get_object(*args, **kwds)
        obj.increment_ui_view_count()
        return obj

    def get(self, request, *args, **kwargs):
        try:
            return super().get(request, *args, **kwargs)
        except Http404 as original:
            if not request.user.is_anonymous:
                raise original
            return redirect('%s?next=%s' % (settings.LOGIN_URL, request.path))

class PlotDeleteView(LoginRequiredMixin, generic.DeleteView):
    model = Plot
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'

    def get_success_url(self):
        return  reverse_lazy('uhepp_vault:collection-detail',
                             args=[self.object.collection.pk])

    def get_queryset(self):
        queryset = Plot.objects.filter(collection__owner=self.request.user)
        return queryset


def plot_download(request, uuid):
    if request.user.is_anonymous:
        queryset = Plot.objects.filter(collection__visibility=PUBLIC_LEVEL)
    else:
        queryset = Plot.objects.filter(
            Q(collection__visibility__gte=INTERNAL_LEVEL) |
            Q(collection__owner=request.user)
        )
    
    plot = get_object_or_404(queryset, uuid=uuid)
    plot.increment_ui_view_count()

    response =  JsonResponse(plot.uhepp)
    response["Content-Disposition"] = f'attachment; filename="{plot}.json"'
    return response

def plot_preview(request, uuid):
    if request.user.is_anonymous:
        queryset = Plot.objects.filter(collection__visibility=PUBLIC_LEVEL)
    else:
        queryset = Plot.objects.filter(
            Q(collection__visibility__gte=INTERNAL_LEVEL) |
            Q(collection__owner=request.user)
        )

    plot = get_object_or_404(queryset, uuid=uuid)

    if not plot.preview:
        return HttpResponseRedirect(static('uhepp_vault/preview-loading.png'))
    elif plot.preview.tobytes() == b"!error":
        return HttpResponseRedirect(static('uhepp_vault/preview-error.png'))
    else:
        return HttpResponse(plot.preview, content_type="image/png")

def logout(request):
    if request.user.is_authenticated:
        return render(request, "uhepp_vault/logout.html")
    else:
        return redirect(reverse("uhepp_vault:home"))
